%% Doing Economics with the Computer
% *Lecture 2: SOLUTIONS* $\\$
% Harris Dellas $\\$
% Lucas Kyriacou

%% Introduction to Matlab: Controlling the Flow
% The objective of this exercise is to help you develop your Matlab skills 
% along two dimensions:  Writing script files. Controlling the flow 
% (the for, if, while commands).

%% Compulsory Readings
% Read chapters 1 - 5 in Attaway (2019). The book is provided on Ilias. 

%% Further Exercises
% Practice by going through the practice questions in chapters 1 - 5 in Attaway (2019). 
% Solutions to the practice questions in chapter 3-5 are available on Ilias. 

%% Housekeeping
% When you write a script, you should include these commands at the start.

clear all % Deletes memory
close all % closes open windows (figures etc.)
clc       % clears command window 

%% 1 Example 1
% Open a new M-File, write the below command and save the file as Lecture_2.m 
% and run it from the Matlab prompt by typing Lecture_2

disp('hello world'); 

%% 1.1 Script M-files
%% Example 2
% Write and run the following script M-file.
% Then type the name of the variables a, b, c in the Matlab prompt and see what
% happens.

a = [1 2];
b = [3 4];
c = a + b;

%% Example 3
% Add comments to the above script

% Define some variables
a = [1 2]; % this is a vector
b = [3 4]; % this is another vector

% Calculate new variables
c = a + b  % add

%% 1.2 Controlling the flow
%% 1.2.1 The for loop
%% Example 4
for i = 1:10; disp(i); end

% equivalently (but more readable):

for i = 1:10
    disp(i); 
end

%% Example 5
for i = 1:10
    x(i)=2^i; 
end; 
x

%% Example 6
for i = 1:10
    x(i)=i^2;
end
x

%% Example 7
A = zeros(5,5); 
for i = 1:5
    for j = 1:5
        A(i,j) = i^j; 
    end
end
A

%% Example 8
for i = 1:2:9
    disp(i); 
end

%% Example 9
y = zeros(10,1); 
for i = [2 4 6 8 10] 
    y(i) = 2^i; 
end
y;

%% In Class Exercise 1: Geometric Series
% Consider the geometric series $x(n) = q^{n-1}$ for n=1,2,3,... .
%% a) 
% Calculate the values of the series for n = 1:10, q = 0.5 and store the
% results in a vector S1.

q=0.5;
for n=1:10
    x(n)=q^(n-1);
end
S1=x;

%% b) 
% Compute the sum $s(n)= 1+ q + q^2 + q^3+ \dots + q^{n-1}$ for q=0.5 and n=10.
% This can be done in several ways, b1 to b3:

%% b1)
% Method 1: A recursive representation: $\\$
% $s(n)=1+q+\dots+ q^{n-2} +q^{n-1} \\$
% $s(n-1)=1+q+\dots+q^{n-2}\\$ 
% $s(n)-s(n-1)= q^{n-1} \rightarrow s(n)=s(n-1)+ q^{n-1}\\$ 
% Try out n=1:10

q=0.5;
s(1)=1;
for n=2:10;
    s(n)=s(n-1)+q^(n-1);
end;
s(end)

%% b2)
% Method 2: $\\$ 
% $q*s(n-1)= q*(1+q+\dots+q^{n-2})= q+q^2 +\dots+q^{n-1}\\$
% $s(n)= 1+q+\dots+ q^{n-2} +q^{n-1}\\$
% $\rightarrow s(n) = 1+ q*s(n-1)$

q=0.5;
s(1)=1;
for n=2:10
    s(n)=q*s(n-1)+1;
end
s(end)

%% b3) 
% Method 3: Apply The standard formula: $(1-q^{n})/(1-q)$

% YOUR CODE HERE

%% c)
% Store the values of the geometric series $x(n)= q^{(n-1)}$ for n = 1:10 and q = 0.2 in the first row of a matrix S. 
% Store the values for q = 0.5 and q = 0.8 in rows 2 and 3.

S=[];

q=0.2;
for n=1:10
    S(1,n)=q^(n-1);
end

q=0.8;
for n=1:10
    S(2,n)=q^(n-1);
end

q=0.8;
for n=1:10
    S(3,n)=q^(n-1);
end

S

%% In Class Exercise 2: Difference Equations
%% i) First order, linear difference equation
% Let $y(t+1) = f*y(t) + w(t+1)$. Assume that f = 0.7, y(1) = 1.5 and w(t) = 0 for all periods t.

%% a) 
% Generate the time path of y for $t=1, 2, \dots, 20$ using a for loop. Plot the time path of y. What happens if f = 1?

y=[];
f=0.7;
y(1)=1.5;
T=20;
w=zeros(21,1);

for t=1:T
    y(t+1)=f*y(t)+w(t+1);
end
plot(y)

%% b) 
% Consider a temporary change in w: $w(4) = 1$. How is the time path of y affected? Compute and plot the impulse response function.

y=[];
f=0.7;
y(1)=1.5;
T=20;
w=zeros(21,1);
w(4)=1;
for t=1:T;
    y(t+1)=f*y(t)+w(t+1);
end;
plot(y)

%% c) 
% Consider a permanent change in w: $w(t) = 1$ for $t =  2, 3, \dots$ How is the time path of y affected? Compute and plot the immediate and future effects.

y=[];
f=0.7;
y(1)=1.5;
T=20;
w=ones(21,1);
w(1:2)=0;
for t=1:T
    y(t+1)=f*y(t)+w(t+1);
end
plot(y);

%% d) 
% Let's assume that w(t) is a stochastic variable, a normally distributed random variable: 
% $w(t): N(0, s^2 = 0.1)$. Generate a sequence $w(t)$ for $t = 2, .., 20$.
% Compute and plot the time path of y. Compare this to the time path under (a).

y=[];
f=0.7;
y(1)=1.5;
T=20;
w=0.1*randn(1,21);
for i=1:T
    y(i+1)=f*y(i)+w(i+1);
end
plot(y)

%% ii) Second order, linear difference equation
%% a)
% Let $y(t+1) = f*y(t) + g*y(t-1)+ w(t+1)$. 
% Assume that f=0.7, g=-0.2, y(1)=1.5, y(2)=1, and w(t)=0 for all periods t. 
% Generate the time path of y for $t = 1, 2, ..., 20$ using a for loop. Plot the time path of y.

y=[];
f=0.7;
g=-0.2;
y(1)=1.5;
y(2)=1;
T=20;
w=zeros(21,1);
for t=2:T
    y(t+1)=f*y(t)+g*y(t-1)+w(t+1);
end
plot(y)

%% iii) 1st order, non-linear difference equation
%% a)
% Consider the non-linear, difference equation $y(t+1) = w*y(t)^{1/\sqrt{(m)}} + k$. 
% Let m=1.5, y(1)=2, w=0.5 and k=1. Suppose that when t=10, k becomes 2 and remains at this level thereafter. 
% Plot the path of the difference equation over 50 periods.

y=[];
k=[];
m=1.5;
y(1)=2;
T=50;
k(1:9)=1;
k(10:50)=2;
w=0.55;
for t=1:T
    y(t+1)=w*y(t)^(1/sqrt(m))+k(t);
end
plot(y)

%% b)
% Consider the non-linear, difference equation $y(t+1) =  w*y(t)^{1/\sqrt{(m)}} + k$ 
% where w is a random number from a uniform distribution. 
% Let m=1.5, y(1)=2, and k=1. Suppose that when t=10, k becomes 2 and remains at this level thereafter. 
% Plot the path of the difference equation over 50 periods.

y=[];
k=[];
m=1.5;
y(1)=2;
T=50;
k(1:9)=1;
k(10:50)=2;
w=rand(1,50);

for t=1:T
    y(t+1)=w(t)*y(t)^(1/sqrt(m))+k(t);
end
plot(y)

%% 1.2.2 The if statement
%% Example 10
a = 4; 
b = 3;

if a>b
    disp('a is bigger than b'); 
end

%% Example 11
a = 2; 
b = 3; 

if a>b
    disp('a is bigger than b');
else 
    disp('a is smaller or equal to b');
end

%% Example 12
a = 2; 
b = 3; 

if a>b
    disp('a is bigger than b');
elseif a == b
    disp('a is equal to b');
else
    disp('a is smaller than b'); 
end

%% Example 13
a = 2; b = 3; c = 3;

if (a < b) && (b  ~= c)
    disp('a is smaller than b and b is not equal to c');
end

%% Example 14
x(1) = 1; 
for i = 1:1000
    x(i+1) = x(i)*2;
    if x(i+1) > 100
        break
    end
end
x
%% In Class Exercise 3
% The sum of (infinite) geometric series:
% Calculate the sum of a geometric series $s(n)= 1+ q + q^2 + q^3+ \dots + q^{n-1}$ for q=0.5 
% and $n\rightarrow\infty$ using a for loop. Stop the loop when the value of s(n) equals approximately s(n-1). 
% Example 14 gives an idea of how to do it.

s=[]; tol=1e-05; crit=1; s(1)=1; q=0.5;

for n=1:10000
    s(n+1)=s(n)+q^n;
    crit=abs(s(n+1)-s(n));
    if crit<tol
        break
    end
end
size(s)

%% In Class Exercise 4: The solution to a linear difference equation
% The difference equation $y(t+1) = f*y(t) + w(t+1)$ converges towards a 
% certain value when w is a constant and $-1<f<1$. 
% Calculate this value when y(1)=1, w=1 using a for loop 
% (terminate the loop when the value of y(t) no longer changes.

N=10000; f=0.7; y(1)=1; w=ones(1,N); tol=1e-5; crit=1;
for i=1:N
    y(i+1)=f*y(i)+w(i+1);
    crit=abs(y(i+1)-y(i));
    if crit<tol
        break;
    end
end
y(end)

%% In Class Exercise 5
% Consider the non-linear difference equation, $y(t+1) = f*(y(t))^{0.3}- k*(y(t-1))^{-0.2} +w$. 
% Let f=0.5, k=0.4, w=1. Use an if loop in order to get to find the value 
% this equation converges to when $t\rightarrow\infty$.

N=1000; f=0.5; k=0.4; y(1)=1; y(2)=1; w=1; tol=1e-5; crit=1;
for t=2:N
    y(t+1) = f*(y(t))^(0.3)- k*(y(t-1))^(-0.2)+w;
    crit=abs(y(t+1)-y(t));
    if crit<tol
        break;
    end
end
y(end)

%% 1.2.3 The while loop
%% Example 15
i = 0; 
while i<=9
    i = i+1; 
    disp(i); 
end

%% Example 16
x = 1; 
while x<=100
    x = x*2;
end
x

%% In Class Exercise 6
% Redo In Class Exercise 4 using a while loop in order to get the value at which the change in
% successive values of the difference equation is ''negligible''. Example 16 may help.

tol=1e-5; crit=1; x0=1; w=1; N=1; phi=0.7; 
while crit>tol
    x=phi*x0+w;
    crit=abs(x-x0);
    x0=x;
    N=N+1;
end

%% In Class Exercise 7
% Consider the non-linear difference equation, $y(t+1) = f*\sqrt{y(t)}+ w$. Let f=0.5, y_1= 1, w=1. 
% Use a while loop in order to get the value at which the change in successive values of the difference equation is ''negligible''.

tol=1e-5; crit=1; x0=1; w=1; f=0.7;
while crit>tol
    x=f*sqrt(x0)+w;
    crit=abs(x-x0);
    x0=x;
end

%% Note
tol=1e-5; crit=1; x0=1; q=0.5; 
while crit>tol
    x=q*x0+1;
    crit=abs(x-x0);
    x0=x;
end

% x(n)=1+q+...+q^(n-1)
% x(n-1)=1+q+...+q^(n-2)
% q*x(n-1)=q+...+q^(n-1) = x(n)-1
% x(n) = q*x(n-1)+1